/*
    SPDX-FileCopyrightText: 1998-2001 Andreas Zehender <az@azweb.de>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#ifndef KSPACEDUEL_MATHROUTINES_H
#define KSPACEDUEL_MATHROUTINES_H

double rectToAngle(double x,double y);
double average(double phi1,double phi2);
double difference(double phi1,double phi2);

#endif // KSPACEDUEL_MATHROUTINES_H
